﻿unit desktop.sidemenu;

interface

uses
  qtx.sysutils, qtx.classes, qtx.dom.widgets, qtx.dom.types,
  qtx.delegates;

type
 TQTXSideMenuList = class;
 TQTXSideMenuConstructor = procedure (Menu: TQTXSideMenuList);

 TQTXSideMenuItem = class;
 TQTXSideMenuItemConstructor = procedure(Item: TQTXSideMenuItem);

 TQTXSideMenuItem = class
 protected
  FOnClick: TNotifyEvent;
  FAnchor: JElement;
  FLink: JElement;
  FTitle: String;
  FIcon: String;

  function GetVisible: Boolean;
  procedure SetVisible(const AValue: Boolean);

  function GetString(AIndex: integer): String;
  procedure SetString(AIndex: integer; AValue: String);
 public
  constructor Create(AParent: THandle);

  property Title: String index 0 read GetString write SetString;
  property Icon: String index 1 read GetString write SetString;

  property OnClick: TNotifyEvent read FOnClick write FOnClick;
  property Visible: Boolean read GetVisible write SetVisible;
 end;

 TQTXSideMenuList = class(TQTXWidget)
  protected
   FItems: Array of TQTXSideMenuItem;
   FMenuHandle: THandle;
   FBottomMenuHandle: THandle;
   function CreateElementInstance: TWidgetHandle;override;
   function GetRightMenu: Boolean;
   procedure SetRightMenu(const AValue: Boolean);
  public
   constructor Create(AOwner: TQTXComponent; CB: TQTXSideMenuConstructor); reintroduce; virtual;
   procedure Clear;
   function Add(const ABottom: Boolean; const AIcon, ATitle: String; const AOnClick: TNotifyEvent): TQTXSideMenuItem;

   property  RightMenu: Boolean read GetRightMenu write SetRightMenu;
   property  Items[index: int32]: TQTXSideMenuItem read ( FItems[index] );
   property  ItemCount: integer read (FItems.length);
  end;

implementation uses qtx.dom.stylesheet;

var document external "document": JDocument;

constructor TQTXSideMenuItem.Create(AParent: THandle);
begin
  FLink := document.createElement('li');

  FAnchor := document.createElement("a");
  FAnchor.addEventListener('click', procedure(Aevent: JEvent)
  begin
   if assigned(FOnClick) then FOnClick(Self);
  end);
  FLink.appendChild(FAnchor);
  AParent.appendChild(FLink);
end;

function TQTXSideMenuItem.GetString(AIndex: integer): String;
begin
 case AIndex of
  0: result := FTitle;
  1: result := FIcon;
  else result := '';
 end;
end;

procedure TQTXSideMenuItem.SetString(AIndex: integer; AValue: String);
begin
 case AIndex of
  0: FTitle := AValue;
  1: FIcon := AValue;
 end;
 FAnchor.innerHTML := '<i class="fa '+FIcon+' fa-2x"></i>'+
  '<span class="nav-text">'+FTitle+'<span>';
end;

function TQTXSideMenuItem.GetVisible: Boolean;
begin
 result := FLink.classList.contains('hide');
end;

procedure TQTXSideMenuItem.SetVisible(const AValue: Boolean);
begin
 if AValue then
  FLink.classList.remove('hide')
 else
  FLink.classList.add('hide');
end;


function TQTXSideMenuList.CreateElementInstance: TWidgetHandle;
begin
 result := document.createElement('nav');
 result["type"] := "nav";
end;

constructor TQTXSideMenuList.Create(AOwner: TQTXComponent; CB: TQTXSideMenuConstructor);
begin
  inherited Create(AOwner, procedure(Widget: TQTXWidget)
  begin
   FMenuHandle := document.createElement('ul');
   FBottomMenuHandle := document.createElement('ul');
   FBottomMenuHandle.classList.add("logout");

   Widget.Handle.appendChild(FMenuHandle);
   Widget.Handle.appendChild(FBottomMenuHandle);
   PositionMode := TQTXWidgetPositionMode.cpAbsolute;

   if assigned(CB) then CB(self);
  end);
end;

procedure TQTXSideMenuList.SetRightMenu(const AValue: Boolean);
begin
 if AValue then
  CssClasses.ClassAdd('menu-right') else
  CssClasses.ClassRemove('menu-right')
end;

function TQTXSideMenuList.GetRightMenu: Boolean;
begin
 result := CssClasses.ClassExists('menu-right');
end;

function TQTXSideMenuList.Add(const ABottom: Boolean; const AIcon, ATitle: String; const AOnClick: TNotifyEvent): TQTXSideMenuItem;
begin
 if ABottom then
  result := TQTXSideMenuItem.create(FBottomMenuHandle)
 else
  result := TQTXSideMenuItem.create(FMenuHandle);

 result.Icon := AIcon;
 result.Title := ATitle;
 result.OnClick := AOnClick;

 FItems.Add(result);
end;

procedure TQTXSideMenuList.Clear;
begin
 for var i := 0 to FItems.Length-1 do
  FItems[i].Free;
 FItems.Clear;
end;

initialization
begin
 var lSheet := TQTXCSSRules.Global;

 lSheet.AddStyles(#'
    .TQTXSideMenuList .fa-2x,
    .TQTXSideMenuList .fa {
        position: relative;
        display: table-cell;
        width: 60px;
        height: 36px;
        text-align: center;
        vertical-align: middle;
        font-size: 2em;
    }

    .TQTXSideMenuList:hover {
        width: 250px;
        overflow: visible;
    }

    .TQTXSideMenuList {
        background: #212121;
       /* background: linear-gradient(to left, rgba(21,21,21,1) 0%, rgba(21,21,21,0.5) 100%);*/
       /* border-right: 1px solid #e5e5e5; */
        position: absolute;
        top: 0;
        bottom: 0;
        height: 100%;
        left: 0;
        width: 60px;
        overflow: hidden;
        -webkit-transition: width 0.05s linear;
        transition: width 0.05s linear;
        -webkit-transform: translateZ(0) scale(1, 1);
        -webkit-user-select: none;
        -moz-user-select: none;
        -ms-user-select: none;
        -o-user-select: none;
        user-select: none;
    }

    .TQTXSideMenuList ul {
        margin: 7px 0;
    }

    .TQTXSideMenuList li {
        position: relative;
        display: block;
        width: 250px;
    }

    .TQTXSideMenuList a {
        position: relative;
        display: table;
        border-collapse: collapse;
        border-spacing: 0;
        color: #999;
        font-family: arial;
        font-size: 14px;
        text-decoration: none;
        -webkit-transform: translateZ(0) scale(1, 1);
        -webkit-transition: all 0.1s linear;
        transition: all 0.1s linear;
    }

    .TQTXSideMenuList .nav-icon,
    .TQTXSideMenuList .fa {
        font-size: 18px;
    }

    .TQTXSideMenuList .nav-text {
        display: table-cell;
        vertical-align: middle;
        width: 190px;
    }

    .TQTXSideMenuList .logout {
        position: absolute;
        left: 0;
        bottom: 0;
    }

    .TQTXSideMenuList a:hover,
    .TQTXSideMenuList a:focus {
        text-decoration: none;
    }

    .TQTXSideMenuList ul,
    .TQTXSideMenuList li {
        outline: 0;
        margin: 0;
        padding: 0;
    }

    .TQTXSideMenuList li:hover a,
    .TQTXSideMenuList li:focus a {
        color: #fff;
        background-color: #000000;
    }

    .TQTXSideMenuList li.hidden {
       display: none;
    }

    .TQTXSideMenuList .hide {
     display: none;
    }

    .TQTXSideMenuList.menu-right {
      position: absolute;
      left: auto;
      right: 0;
      text-align: right;
    }

    .TQTXSideMenuList.menu-right .nav-text {
       text-align: left;
    }
  ');
end;
end.
